@@ -1,10 +1,6 @@
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import absolute_import, division, print_function
-
-__metaclass__ = type
-
-from ansible_collections.community.aws.tests.unit.compat.mock import MagicMock
+from unittest.mock import MagicMock
 from ansible.utils.path import unfrackpath
 
 
