@@ -1,8 +1,9 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# Copyright: (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
-# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
+# Copyright (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
+# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
@@ -17,13 +18,20 @@ requirements:
 short_description: Manage Cloudflare DNS records
 description:
    - "Manages dns records via the Cloudflare API, see the docs: U(https://api.cloudflare.com/)."
+extends_documentation_fragment:
+   - community.general.attributes
+attributes:
+  check_mode:
+    support: full
+  diff_mode:
+    support: none
 options:
   api_token:
     description:
     - API token.
     - Required for api token authentication.
     - "You can obtain your API token from the bottom of the Cloudflare 'My Account' page, found here: U(https://dash.cloudflare.com/)."
-    - Can be specified in C(CLOUDFLARE_TOKEN) environment variable since community.general 2.0.0.
+    - Can be specified in E(CLOUDFLARE_TOKEN) environment variable since community.general 2.0.0.
     type: str
     required: false
     version_added: '0.2.0'
@@ -43,39 +51,39 @@ options:
   algorithm:
     description:
     - Algorithm number.
-    - Required for C(type=DS) and C(type=SSHFP) when C(state=present).
+    - Required for O(type=DS) and O(type=SSHFP) when O(state=present).
     type: int
   cert_usage:
     description:
     - Certificate usage number.
-    - Required for C(type=TLSA) when C(state=present).
+    - Required for O(type=TLSA) when O(state=present).
     type: int
     choices: [ 0, 1, 2, 3 ]
   hash_type:
     description:
     - Hash type number.
-    - Required for C(type=DS), C(type=SSHFP) and C(type=TLSA) when C(state=present).
+    - Required for O(type=DS), O(type=SSHFP) and O(type=TLSA) when O(state=present).
     type: int
     choices: [ 1, 2 ]
   key_tag:
     description:
     - DNSSEC key tag.
-    - Needed for C(type=DS) when C(state=present).
+    - Needed for O(type=DS) when O(state=present).
     type: int
   port:
     description:
     - Service port.
-    - Required for C(type=SRV) and C(type=TLSA).
+    - Required for O(type=SRV) and O(type=TLSA).
     type: int
   priority:
     description:
     - Record priority.
-    - Required for C(type=MX) and C(type=SRV)
+    - Required for O(type=MX) and O(type=SRV)
     default: 1
     type: int
   proto:
     description:
-    - Service protocol. Required for C(type=SRV) and C(type=TLSA).
+    - Service protocol. Required for O(type=SRV) and O(type=TLSA).
     - Common values are TCP and UDP.
     - Before Ansible 2.6 only TCP and UDP were available.
     type: str
@@ -83,30 +91,30 @@ options:
     description:
     - Proxy through Cloudflare network or just use DNS.
     type: bool
-    default: no
+    default: false
   record:
     description:
     - Record to add.
-    - Required if C(state=present).
-    - Default is C(@) (e.g. the zone name).
+    - Required if O(state=present).
+    - Default is V(@) (that is, the zone name).
     type: str
     default: '@'
     aliases: [ name ]
   selector:
     description:
     - Selector number.
-    - Required for C(type=TLSA) when C(state=present).
+    - Required for O(type=TLSA) when O(state=present).
     choices: [ 0, 1 ]
     type: int
   service:
     description:
     - Record service.
-    - Required for I(type=SRV).
+    - Required for O(type=SRV).
     type: str
   solo:
     description:
     - Whether the record should be the only one for that record type and record name.
-    - Only use with C(state=present).
+    - Only use with O(state=present).
     - This will delete all other records with the same record name and type.
     type: bool
   state:
@@ -128,20 +136,20 @@ options:
     default: 1
   type:
     description:
-      - The type of DNS record to create. Required if C(state=present).
-      - C(type=DS), C(type=SSHFP) and C(type=TLSA) added in Ansible 2.7.
+      - The type of DNS record to create. Required if O(state=present).
+      - O(type=DS), O(type=SSHFP), and O(type=TLSA) were added in Ansible 2.7.
     type: str
     choices: [ A, AAAA, CNAME, DS, MX, NS, SPF, SRV, SSHFP, TLSA, TXT ]
   value:
     description:
     - The record value.
-    - Required for C(state=present).
+    - Required for O(state=present).
     type: str
     aliases: [ content ]
   weight:
     description:
     - Service weight.
-    - Required for C(type=SRV).
+    - Required for O(type=SRV).
     type: int
     default: 1
   zone:
@@ -205,7 +213,7 @@ EXAMPLES = r'''
     zone: example.net
     type: CNAME
     value: example.com
-    proxied: yes
+    proxied: true
     account_email: test@example.com
     account_api_key: dummyapitoken
     state: present
@@ -303,7 +311,7 @@ record:
             description: No documentation available.
             returned: success
             type: bool
-            sample: False
+            sample: false
         meta:
             description: No documentation available.
             returned: success
@@ -328,12 +336,12 @@ record:
             description: Whether this record can be proxied through Cloudflare.
             returned: success
             type: bool
-            sample: False
+            sample: false
         proxied:
             description: Whether the record is proxied through Cloudflare.
             returned: success
             type: bool
-            sample: False
+            sample: false
         ttl:
             description: The time-to-live for the record.
             returned: success
@@ -360,7 +368,7 @@ import json
 
 from ansible.module_utils.basic import AnsibleModule, env_fallback
 from ansible.module_utils.six.moves.urllib.parse import urlencode
-from ansible.module_utils._text import to_native, to_text
+from ansible.module_utils.common.text.converters import to_native, to_text
 from ansible.module_utils.urls import fetch_url
 
 
@@ -605,7 +613,7 @@ class CloudflareAPI(object):
                 content = str(params['key_tag']) + '\t' + str(params['algorithm']) + '\t' + str(params['hash_type']) + '\t' + params['value']
         elif params['type'] == 'SSHFP':
             if not (params['value'] is None or params['value'] == ''):
-                content = str(params['algorithm']) + '\t' + str(params['hash_type']) + '\t' + params['value']
+                content = str(params['algorithm']) + ' ' + str(params['hash_type']) + ' ' + params['value'].upper()
         elif params['type'] == 'TLSA':
             if not (params['value'] is None or params['value'] == ''):
                 content = str(params['cert_usage']) + '\t' + str(params['selector']) + '\t' + str(params['hash_type']) + '\t' + params['value']
@@ -686,10 +694,11 @@ class CloudflareAPI(object):
                 "port": params['port'],
                 "weight": params['weight'],
                 "priority": params['priority'],
-                "name": params['record'][:-len('.' + params['zone'])],
+                "name": params['record'],
                 "proto": params['proto'],
                 "service": params['service']
             }
+
             new_record = {"type": params['type'], "ttl": params['ttl'], 'data': srv_data}
             search_value = str(params['weight']) + '\t' + str(params['port']) + '\t' + params['value']
             search_record = params['service'] + '.' + params['proto'] + '.' + params['record']
@@ -717,7 +726,7 @@ class CloudflareAPI(object):
                 if (attr is None) or (attr == ''):
                     self.module.fail_json(msg="You must provide algorithm, hash_type and a value to create this record type")
             sshfp_data = {
-                "fingerprint": params['value'],
+                "fingerprint": params['value'].upper(),
                 "type": params['hash_type'],
                 "algorithm": params['algorithm'],
             }
@@ -727,7 +736,7 @@ class CloudflareAPI(object):
                 'data': sshfp_data,
                 "ttl": params['ttl'],
             }
-            search_value = str(params['algorithm']) + '\t' + str(params['hash_type']) + '\t' + params['value']
+            search_value = str(params['algorithm']) + ' ' + str(params['hash_type']) + ' ' + params['value']
 
         if params['type'] == 'TLSA':
             for attr in [params['port'], params['proto'], params['cert_usage'], params['selector'], params['hash_type'], params['value']]:
